<?php
/**
 *
 * ShipperHQ
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@magentocommerce.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade Magento to newer
 * versions in the future. If you wish to customize Magento for your
 * needs please refer to http://www.magentocommerce.com for more information.
 *
 * Shipper HQ Shipping
 *
 * @category ShipperHQ
 * @package ShipperHQ_Common
 * @copyright Copyright (c) 2015 Zowta LLC (http://www.ShipperHQ.com)
 * @license http://opensource.org/licenses/osl-3.0.php Open Software License (OSL 3.0)
 * @author ShipperHQ Team sales@shipperhq.com
 */
/**
 * Copyright © 2015 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */
namespace ShipperHQ\Common\Model\Checkout;

use ShipperHQ\Lib\Checkout\AbstractService;

/**
 * Class Service
 *
 * @package ShipperHQ_Common
 */
class Service extends AbstractService
{
    /**
     * @var \Magento\Checkout\Model\Session
     */
    protected $checkoutSession;

    /**
     * Quote repository.
     *
     * @var \Magento\Quote\Api\CartRepositoryInterface
     */
    protected $quoteRepository;
    /**
     * Shipping method converter
     *
     * @var \Magento\Quote\Model\Cart\ShippingMethodConverter
     */
    protected $converter;
    /**
     * @var \ShipperHQ\Shipper\Helper\LogAssist
     */
    private $shipperLogger;

    protected $address;
    protected $quote;

    /**
     * @param \Magento\Quote\Api\CartRepositoryInterface $quoteRepository
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Magento\Quote\Model\Cart\ShippingMethodConverter $converter
     * @param \ShipperHQ\Shipper\Helper\LogAssist $shipperLogger
     * @codeCoverageIgnore
     */
    public function __construct(
        \Magento\Quote\Api\CartRepositoryInterface $quoteRepository,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Quote\Model\Cart\ShippingMethodConverter $converter,
        \ShipperHQ\Shipper\Helper\LogAssist $shipperLogger
    ) {
        $this->quoteRepository = $quoteRepository;
        $this->checkoutSession = $checkoutSession;
        $this->converter = $converter;
        $this->shipperLogger = $shipperLogger;
    }

    /*
     * Cache data selected at checkout for use in rate request
     */
    public function saveSelectedData($data)
    {
        $requestData = $this->checkoutSession->getShipperhqData();
        $requestData['checkout_selections'] = $data;
        $this->checkoutSession->setShipperhqData($requestData);
    }
    /*
     * Remove carrier shipping rates before re-requesting
     */
    public function cleanDownRates($cartId, $carrierCode, $carriergroupId, $addressId = false)
    {
        if (is_null($cartId)) {
            return;
        }
        $currentRates = $this->getAddress($cartId, $addressId)->getGroupedAllShippingRates();
        foreach ($currentRates as $code => $rates) {
            //SHQ18-57 removed comparison of code and carrier group before removing rates. Since all rates are now refreshed.
            foreach ($rates as $rate) {
                    $rate->isDeleted(true);
            }
        }
    }

    /*
     * Request shipping rates for specified carrier
     */
    public function reqeustShippingRates($cartId, $carrierCode, $carriergroupId, $addressData, $addressId = false)
    {
        $address = $this->getAddress($cartId, $addressId);
        $address->addData($addressData);
        $rateFound = $address->requestShippingRates();
        $rates = $address->getGroupedAllShippingRates();
        $output = [];
        foreach ($rates as $carrierRates) {
            foreach ($carrierRates as $rate) {
                $rateObject = $this->converter->modelToDataObject(
                    $rate,
                    $this->getQuote($cartId)->getQuoteCurrencyCode()
                );
                //mimicking output format from serviceOutputProcessor->process(... (API stuff)
                $oneRate = ['carrier_code' => $rateObject->getCarrierCode(),
                            'method_code' => $rateObject->getMethodCode(),
                            'carrier_title' => $rateObject->getCarrierTitle(),
                            'method_title' => $rateObject->getMethodTitle(),
                            'amount'    => $rateObject->getAmount(),
                            'base_amount' => $rateObject->getBaseAmount(),
                            'available' => $rateObject->getAvailable(),
                            'error_message' => $rateObject->getErrorMessage(),
                            'price_excl_tax' => $rateObject->getPriceExclTax(),
                            'price_incl_tax' => $rateObject->getPriceInclTax()];

                $output[] = $oneRate;
            }
        }
        return $output;
    }

    /*
     * Removed cached data selected at checkout
     */
    public function cleanDownSelectedData()
    {
        $requestData = $this->checkoutSession->getShipperhqData();
        unset($requestData['checkout_selections']);
        $this->checkoutSession->setShipperhqData($requestData);
    }

    protected function saveShippingAddress($cartId, $addressId)
    {
        $address = $this->getAddress($cartId, $addressId);
        $region = $address->getRegion();
        if ($region !== null && $region instanceof \Magento\Customer\Model\Data\Region) {
            $regionString = $region->getRegion();
            $address->setRegion($regionString);
        }
        try {
            $address->save();
        } catch (\Exception $e) {
            $this->shipperLogger->postCritical(
                'Shipperhq_Shipper',
                'Exception raised whilst saving shipping address',
                $e->getMessage()
            );
        }
    }

    protected function getAddress($cartId, $addressId = false)
    {

        if ($this->address === null) {
            if (!$this->quote) {
                $this->quote = $this->getQuote($cartId);
            }
            $this->address = $this->quote->getShippingAddress();
        }
        //TODO multiaddress checkout support
//        if($addressId) {
//            $allShipAddress = $this->quote->getAllShippingAddresses();
//            foreach($allShipAddress as $shippingAddress) {
//                if($shippingAddress->getId() == $addressId) {
//                    $this->_address = $shippingAddress;
//                }
//            }
//        }

        return $this->address;
    }

    protected function getQuote($cartId)
    {
        if (!$this->quote) {
            $this->quote = $this->quoteRepository->getActive($cartId);
        }
        return $this->quote;
    }
}
